<?php
/**
 * This is the entrypoint for the locrMAPS WordPress plugin.
 *
 * @package locrMAPS
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects;

use Locr\WordPress\Plugin\LocrMaps\DDD\ValueObjects\IconOptions\{IconColor, IconName, SetIconOnInitialPosition};
use Locr\WordPress\Plugin\LocrMaps\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the options.
 *
 * @property-read IconName $icon_name The icon name.
 * @property-read SetIconOnInitialPosition $set_icon_on_initial_position The set icon on initial position.
 */
final class IconOptions {
	public const OPTION_NAME = Plugin::SLUG . '_icon_options';

	/**
	 * The icon name.
	 *
	 * @var IconName
	 */
	private IconName $icon_name;
	/**
	 * The set icon on initial position.
	 *
	 * @var SetIconOnInitialPosition
	 */
	private SetIconOnInitialPosition $set_icon_on_initial_position;

	/**
	 * The constructor.
	 *
	 * @param array<mixed> $shortcode_attributes The shortcode attributes.
	 * @param string       $only_option The only option.
	 */
	public function __construct( array $shortcode_attributes = array(), private string $only_option = '' ) {
		$this->icon_name                    = new IconName();
		$this->set_icon_on_initial_position = new SetIconOnInitialPosition();

		$this->parse_options();
		$this->parse_attributes( $shortcode_attributes );
	}

	/**
	 * The magick getter.
	 *
	 * @param string $name The name of the property.
	 * @throws \InvalidArgumentException If the property does not exist.
	 */
	public function __get( string $name ): mixed {
		return match ( $name ) {
			'icon_name' => $this->icon_name,
			'set_icon_on_initial_position' => $this->set_icon_on_initial_position,
			default => throw new \InvalidArgumentException( "Property $name does not exist" ), // phpcs:ignore
		};
	}

	/**
	 * This function parses the attributes.
	 *
	 * @param array<mixed> $shortcode_attributes The shortcode attributes.
	 */
	private function parse_attributes( array $shortcode_attributes = array() ): void {
		if ( ( empty( $this->only_option ) || IconName::OPTION_KEY === $this->only_option ) && isset( $shortcode_attributes[ IconName::ATTRIBUTE_KEY ] ) && is_string( $shortcode_attributes[ IconName::ATTRIBUTE_KEY ] ) ) {
			$this->icon_name = new IconName( trim( $shortcode_attributes[ IconName::ATTRIBUTE_KEY ] ) );
		}
		if ( ( empty( $this->only_option ) || SetIconOnInitialPosition::OPTION_KEY === $this->only_option ) && isset( $shortcode_attributes[ SetIconOnInitialPosition::ATTRIBUTE_KEY ] ) ) {
			$this->set_icon_on_initial_position = new SetIconOnInitialPosition( boolval( $shortcode_attributes[ SetIconOnInitialPosition::ATTRIBUTE_KEY ] ) );
		}
	}

	/**
	 * This function parses the options.
	 */
	private function parse_options(): void {
		$options = get_option( self::OPTION_NAME );
		if ( ! is_array( $options ) ) {
			return;
		}

		if ( ( empty( $this->only_option ) || IconName::OPTION_KEY === $this->only_option ) && isset( $options[ IconName::OPTION_KEY ] ) && is_string( $options[ IconName::OPTION_KEY ] ) ) {
			$this->icon_name = new IconName( trim( $options[ IconName::OPTION_KEY ] ) );
		}
		if ( ( empty( $this->only_option ) || SetIconOnInitialPosition::OPTION_KEY === $this->only_option ) && isset( $options[ SetIconOnInitialPosition::OPTION_KEY ] ) ) {
			$this->set_icon_on_initial_position = new SetIconOnInitialPosition( boolval( $options[ SetIconOnInitialPosition::OPTION_KEY ] ) );
		}
	}

	/**
	 * This function returns the options as an array.
	 *
	 * @return array<mixed>
	 */
	public function to_array(): array {
		return array(
			IconName::ATTRIBUTE_KEY                 => (string) $this->icon_name,
			SetIconOnInitialPosition::ATTRIBUTE_KEY => $this->set_icon_on_initial_position->get_value(),
		);
	}

	/**
	 * This function returns the options as JSON.
	 */
	public function to_json(): string {
		$json = wp_json_encode( $this->to_array() );
		if ( false === $json ) {
			return '';
		}

		return $json;
	}
}
